@@ -20,26 +20,28 @@ class HTTPSignatureComponentResolver:
 
     # TODO: describe interface
     def __init__(self, message):
+        self.message = message
         self.message_type = "request"
         if hasattr(message, "status_code"):
             self.message_type = "response"
-        self.method = getattr(message, "method", None)
         self.url = message.url
-        self.status_code = getattr(message, "status_code", None)
-
         # TODO: check header key and value transforms are applied per 2.1
         self.headers = CaseInsensitiveDict(message.headers)
 
     def resolve(self, component_id):
         if component_id.startswith("@"):  # derived component
             if component_id not in self.derived_component_names:
-                raise HTTPMessageSignaturesException(f'Unknown derived component name "{component_id}"')
+                raise HTTPMessageSignaturesException(f'Unknown covered derived component name "{component_id}"')
             resolver = getattr(self, "get_" + component_id[1:].replace("-", "_"))
             return resolver()
+        if component_id not in self.headers:
+            raise HTTPMessageSignaturesException(f'Covered header field "{component_id}" not found in the message')
         return self.headers[component_id]
 
     def get_method(self):
-        return self.method.upper()
+        if self.message_type == "response":
+            return self.message.request.method.upper()
+        return self.message.method.upper()
 
     def get_target_uri(self):
         return self.url
@@ -67,7 +69,7 @@ class HTTPSignatureComponentResolver:
     def get_status(self):
         if self.message_type != "response":
             raise HTTPMessageSignaturesException('Unexpected "@status" component in a request signature')
-        return str(self.status_code)
+        return str(self.message.status_code)
 
     def get_request_response(self):
         raise NotImplementedError()
