@@ -58,7 +58,7 @@ class HTTPSignatureHandler:
 class HTTPMessageSigner(HTTPSignatureHandler):
     DEFAULT_SIGNATURE_LABEL = "pyhms"
 
-    def sign(self, request, *,
+    def sign(self, message, *,
              key_id: str,
              created: datetime.datetime = None,
              expires: datetime.datetime = None,
@@ -80,7 +80,7 @@ class HTTPMessageSigner(HTTPSignatureHandler):
             signature_params["nonce"] = nonce
         if include_alg:
             signature_params["alg"] = self.signature_algorithm.algorithm_id
-        sig_base, sig_params_node, _ = self.build_signature_base(request,
+        sig_base, sig_params_node, _ = self.build_signature_base(message,
                                                                  covered_component_ids=covered_component_ids,
                                                                  signature_params=signature_params)
         signer = self.signature_algorithm(private_key=key)
@@ -89,25 +89,31 @@ class HTTPMessageSigner(HTTPSignatureHandler):
         if label is not None:
             sig_label = label
         sig_input_node = http_sfv.Dictionary({sig_label: sig_params_node})
-        request.headers["Signature-Input"] = str(sig_input_node)
+        message.headers["Signature-Input"] = str(sig_input_node)
         sig_node = http_sfv.Dictionary({sig_label: signature})
-        request.headers["Signature"] = str(sig_node)
+        message.headers["Signature"] = str(sig_node)
 
 
 VerifyResult = collections.namedtuple("VerifyResult", "label algorithm covered_components parameters body")
 
 
 class HTTPMessageVerifier(HTTPSignatureHandler):
-    def verify(self, response):
-        if "Signature-Input" not in response.headers:
-            raise InvalidSignature("Expected Signature-Input header to be present")
-        sig_inputs = http_sfv.Dictionary()
-        sig_inputs.parse(response.headers["Signature-Input"].encode())
+    def parse_dict_header(self, header_name, headers):
+        if header_name not in headers:
+            raise InvalidSignature(f'Expected "{header_name}" header field to be present')
+        try:
+            dict_header_node = http_sfv.Dictionary()
+            dict_header_node.parse(headers[header_name].encode())
+        except Exception as e:
+            raise InvalidSignature(f'Malformed structured header field "{header_name}"') from e
+        return dict_header_node
+
+    def verify(self, message):
+        sig_inputs = self.parse_dict_header("Signature-Input", message.headers)
         if len(sig_inputs) != 1:
             # TODO: validate all behaviors with multiple signatures
             raise InvalidSignature("Multiple signatures are not supported")
-        signature = http_sfv.Dictionary()
-        signature.parse(response.headers["Signature"].encode())
+        signature = self.parse_dict_header("Signature", message.headers)
         verify_results = []
         for label, sig_input in sig_inputs.items():
             # see 3.2.1, app requirements
@@ -123,11 +129,14 @@ class HTTPMessageVerifier(HTTPSignatureHandler):
             for param in sig_input.params:
                 if param not in self.signature_metadata_parameters:
                     raise InvalidSignature(f'Unexpected signature metadata parameter "{param}"')
-            sig_base, sig_params_node, sig_elements = self.build_signature_base(
-                response,
-                covered_component_ids=covered_component_ids,
-                signature_params=sig_input.params
-            )
+            try:
+                sig_base, sig_params_node, sig_elements = self.build_signature_base(
+                    message,
+                    covered_component_ids=covered_component_ids,
+                    signature_params=sig_input.params
+                )
+            except Exception as e:
+                raise InvalidSignature(e) from e
             verifier = self.signature_algorithm(public_key=key)
             raw_signature = signature[label].value
             try:
